@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.105 2006/05/01 17:26:26 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.107 2006/07/24 17:24:20 richb Exp $
  *
  *  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -192,6 +192,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     parenthesis
 },
 {
+    /* Note to translators: Bksp is short for Backspace. */
     N_("Bksp"),
     N_("Remove rightmost character from displayed value"),
     N_("Backspace"),
@@ -216,6 +217,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     clear
 },
 {
+    /* Note to translators: Clr is short for Clear. */
     N_("Clr"),
     N_("Clear displayed value and any partial calculation"),
     N_("Clear"),
@@ -502,6 +504,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     immediate | postfixop
 },
 {
+    /* Note to translators: Abs is short for Absolute. */
     N_("Abs"),
     N_("Absolute value [u]"),
     NULL,
@@ -1228,7 +1231,8 @@ doerr(char *errmes)
     if (!v->started) {
         return;
     }
-    STRCPY(v->display, errmes);
+    strncpy(v->display, errmes, MAXLINE - 1);
+    v->display[MAXLINE - 1] = '\0';
     set_error_state(TRUE);
     set_display(v->display, FALSE);
     beep();
