@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.h,v 1.8 2006/01/09 16:30:20 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.h,v 1.9 2006/05/01 17:26:26 richb Exp $
  *
  *  Copyright (C) 2004-2006 Sami Pietila
  *
@@ -28,4 +28,10 @@
 void do_factorial(int *, int *);
 void exp_append(char *text);
 
+struct exprm_state *get_state(void);
+void new_state(void);
+void perform_undo(void);
+void perform_redo(void);
+void clear_undo_history(void);
+
 #endif /*FUNCTIONS_H*/
