@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/mpmath.h,v 1.9 2006/01/09 16:30:20 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/mpmath.h,v 1.11 2006/03/28 19:23:33 richb Exp $
  *
  *  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -60,5 +60,16 @@ void calc_sln(int t[MP_SIZE]);
 void calc_syd(int t[MP_SIZE]);
 void calc_term(int t[MP_SIZE]);
 void calc_rshift(int s[MP_SIZE], int t[MP_SIZE], int times, enum shiftd dir);
+void calc_epowy(int s[MP_SIZE], int t[MP_SIZE]);
+
+/* return true if parameter is integer */
+int
+is_integer(int MPnum[MP_SIZE]);
+
+/* return true if parameter is natural
+   number, that is, a positive integer
+*/
+int
+is_natural(int MPnum[MP_SIZE]);
 
 #endif /*MPMATH_H*/
